@@ -25,19 +25,35 @@
 #
 #  $Id$
 #
-=pod 
-=item NAME
+=pod
+
+=head1 NAME
 
 A image viewer program
 
-=item FEATURES
+=head1 FEATURES
 
 Demonstrates usage of Prima image subsystem, in particular:
 
-- Standard open dialog. Note it's behavior with the multi-frame images.
-- Standard save dialog. Note the graphic filters usage.
-- Image conversion routines.
-- Standard Prima::ImageViewer class.
+=over 4
+
+=item *
+
+Standard open dialog. Note it's behavior with the multi-frame images.
+
+=item *
+
+Standard save dialog. Note the graphic filters usage.
+
+=item *
+
+Image conversion routines.
+
+=item *
+
+Standard L<Prima::ImageViewer> class.
+
+=back
 
 Test the correct implementation of the internal image paint routines,
 in particular on the paletted displays and the representation of 1-bit
@@ -48,6 +64,7 @@ Note the mouse wheel interaction.
 =cut
 
 use strict;
+use warnings;
 use Prima qw(ImageViewer StdDlg MsgBox);
 use Prima::Application name => "IV";
 
@@ -134,8 +151,8 @@ sub menuadd
 					['~Normal ( 100%)' => 'Ctrl+Z' => '^Z' => sub{$_[0]-> IV-> zoom(1.0)}],
 					['~Best fit' => 'Ctrl+Shift+Z' => km::Shift|km::Ctrl|ord('z') => \&zbestfit],
 					[],
-					['abfit' => '~Auto best fit' => sub{
-						zbestfit($_[0]) if $_[0]-> IV-> {autoBestFit} = $_[0]-> menu-> abfit-> toggle;
+					['@abfit' => '~Auto best fit' => sub{
+						zbestfit($_[0]) if $_[0]-> IV-> {autoBestFit} = $_[2];
 					}],
 					[],
 					['25%' => sub{$_[0]-> IV-> zoom(0.25)}],
